@@ -1,7 +1,7 @@
 /*
  * Copyright 2003-2006 Gentoo Foundation
  * Distributed under the terms of the GNU General Public License v2
- * $Header: /var/cvsroot/gentoo-projects/pax-utils/paxelf.c,v 1.42 2006/02/05 17:10:52 solar Exp $
+ * $Header: /var/cvsroot/gentoo-projects/pax-utils/paxelf.c,v 1.51 2006/07/30 18:02:00 vapier Exp $
  *
  * Copyright 2005-2006 Ned Ludd        - <solar@gentoo.org>
  * Copyright 2005-2006 Mike Frysinger  - <vapier@gentoo.org>
@@ -100,7 +100,7 @@ int get_etype(elfobj *elf)
 		type = EGET(EHDR32(elf->ehdr)->e_type);
 	else
 		type = EGET(EHDR64(elf->ehdr)->e_type);
-        return type;
+	return type;
 }
 
 const char *get_elfetype(elfobj *elf)
@@ -110,11 +110,31 @@ const char *get_elfetype(elfobj *elf)
 
 void print_etypes(FILE *stream)
 {
-        int i;
+	int i, wrap = 0;
 	for (i = 0; elf_etypes[i].str; ++i) {
-		fprintf(stream, " (%d) = %s\n", elf_etypes[i].value, elf_etypes[i].str);
+		fprintf(stream, " (%4x) = %-10s", elf_etypes[i].value, elf_etypes[i].str);
+		if (++wrap >= 4) {
+			fprintf(stream, "\n");
+			wrap = 0;
+		}
 	}
+	if (wrap)
+		fprintf(stream, "\n");
 }
+
+int etype_lookup(const char *str)
+{
+	if (*str == 'E') {
+		int i;
+		for (i = 0; elf_etypes[i].str; ++i) {
+			if (strcmp(str, elf_etypes[i].str) == 0)
+				return elf_etypes[i].value;
+		}
+	}
+	return atoi(str);
+}
+
+
 /* translate elf EM_ defines */
 static pairtype elf_emtypes[] = {
 	QUERY(EM_NONE),
@@ -191,6 +211,22 @@ static pairtype elf_emtypes[] = {
 	QUERY(EM_OPENRISC),
 	QUERY(EM_ARC_A5),
 	QUERY(EM_XTENSA),
+	QUERY(EM_VIDEOCORE),
+	QUERY(EM_TMM_GPP),
+	QUERY(EM_NS32K),
+	QUERY(EM_TPC),
+	QUERY(EM_SNP1K),
+	QUERY(EM_ST200),
+	QUERY(EM_IP2K),
+	QUERY(EM_MAX),
+	QUERY(EM_CR),
+	QUERY(EM_F2MC16),
+	QUERY(EM_MSP430),
+	QUERY(EM_BLACKFIN),
+	QUERY(EM_SE_C33),
+	QUERY(EM_SEP),
+	QUERY(EM_ARCA),
+	QUERY(EM_UNICORE),
 	QUERY(EM_NUM),
 	QUERY(EM_ALPHA),
 	{ 0, 0 }
@@ -544,6 +580,56 @@ char *pax_short_pf_flags(unsigned long flags)
 	return buffer;
 }
 
+unsigned long pax_pf2hf_flags(unsigned long paxflags)
+{
+	unsigned long flags = 0;
+	char *pf_flags = pax_short_pf_flags(paxflags);
+	size_t x, len = strlen(pf_flags);
+	for (x = 0; x < len; x++) {
+		switch(pf_flags[x]) {
+			case 'p':
+				flags |= HF_PAX_PAGEEXEC;
+				break;
+			case 'P':
+				flags = (flags & ~HF_PAX_PAGEEXEC) | HF_PAX_SEGMEXEC;
+				break;
+			case 'E':
+				flags |= HF_PAX_EMUTRAMP;
+				break;
+			case 'e':
+				flags = (flags & ~HF_PAX_EMUTRAMP);
+				break;
+			case 'm':
+				flags |= HF_PAX_MPROTECT;
+				break;
+			case 'M':
+				flags = (flags & ~HF_PAX_MPROTECT);
+				break;
+			case 'r':
+				flags |= HF_PAX_RANDMMAP;
+				break;
+			case 'R':
+				flags = (flags & ~HF_PAX_RANDMMAP);
+				break;
+			case 'X':
+				flags |= HF_PAX_RANDEXEC;
+				break;
+			case 'x':
+				flags = (flags & ~HF_PAX_RANDEXEC);
+				break;
+			case 's':
+				flags |= HF_PAX_SEGMEXEC;
+				break;
+			case 'S':
+				flags = (flags & ~HF_PAX_SEGMEXEC) | HF_PAX_PAGEEXEC;
+				break;
+			default:
+				break;
+		}
+	}
+	return flags;
+}
+
 char *gnu_short_stack_flags(unsigned long flags)
 {
 	static char buffer[4];
@@ -597,6 +683,8 @@ int elf_max_pt_load(elfobj *elf)
 	Elf ## B ## _Ehdr *ehdr = EHDR ## B (elf->ehdr); \
 	switch (EGET(ehdr->e_ident[EI_OSABI])) { \
 	case ELFOSABI_NONE: \
+	case ELFOSABI_NETBSD: \
+	case ELFOSABI_FREEBSD: \
 	case ELFOSABI_LINUX: \
 	case ELFOSABI_ARM:     return 2; \
 	case ELFOSABI_OPENBSD: return 7; \
@@ -604,5 +692,33 @@ int elf_max_pt_load(elfobj *elf)
 	MAX_PT_LOAD(32)
 	MAX_PT_LOAD(64)
 
-	return 0;
+	return 1;
 }
+#if 0
+ # define ELFOSABI_NONE           0       /* UNIX System V ABI */
+ # define ELFOSABI_SYSV           0       /* Alias.  */
+ # define ELFOSABI_HPUX           1       /* HP-UX */
+ # define ELFOSABI_NETBSD         2       /* NetBSD.  */
+ # define ELFOSABI_LINUX          3       /* Linux.  */
+ # define ELFOSABI_SOLARIS        6       /* Sun Solaris.  */
+ # define ELFOSABI_AIX            7       /* IBM AIX.  */
+ # define ELFOSABI_IRIX           8       /* SGI Irix.  */
+ # define ELFOSABI_FREEBSD        9       /* FreeBSD.  */
+ # define ELFOSABI_TRU64          10      /* Compaq TRU64 UNIX.  */
+ # define ELFOSABI_MODESTO        11      /* Novell Modesto.  */
+ # define ELFOSABI_OPENBSD        12      /* OpenBSD.  */
+ # define ELFOSABI_ARM            97      /* ARM */
+ # define ELFOSABI_STANDALONE     255     /* Standalone (embedded) application */
+
+ /* These 3 ABIs should be in elf.h but are not.
+  * http://www.caldera.com/developers/gabi/latest/ch4.eheader.html#generic_osabi_values
+  */
+
+ # define ELFOSABI_OPENVMS 13     /* OpenVMS */
+ # define ELFOSABI_NSK     14     /* Hewlett-Packard Non-Stop Kernel */
+ # define ELFOSABI_AROS    15     /* Amiga Research OS */
+
+ #4 reserved for IA32 GNU Mach/Hurd
+ #5 reserved for 86Open common IA32 ABI
+
+#endif
