@@ -1,7 +1,7 @@
 /*
  * Copyright 2005-2006 Gentoo Foundation
  * Distributed under the terms of the GNU General Public License v2
- * $Header: /var/cvsroot/gentoo-projects/pax-utils/porting.h,v 1.15 2006/03/19 02:41:17 solar Exp $
+ * $Header: /var/cvsroot/gentoo-projects/pax-utils/porting.h,v 1.22 2006/06/03 18:25:18 solar Exp $
  *
  * Copyright 2005-2006 Ned Ludd        - <solar@gentoo.org>
  * Copyright 2005-2006 Mike Frysinger  - <vapier@gentoo.org>
@@ -12,6 +12,9 @@
 #ifndef _PORTING_H
 #define _PORTING_H
 
+#define ARRAY_SIZE(arr) (sizeof(arr) / sizeof(*arr))
+#undef __PAX_UTILS_CLEANUP
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <errno.h>
@@ -28,6 +31,7 @@
 #include <libgen.h>
 #include <ctype.h>
 #include <pwd.h>
+#include <fnmatch.h>
 
 #include <sys/mman.h>
 #include "elf.h"
@@ -38,10 +42,24 @@
 # include <sys/endian.h>
 #endif
 
-#if defined(__APPLE__) || defined(__NetBSD__) || defined(__OpenBSD__)
-# define SET_STDOUT(fp) err("Darwin/NetBSD/OpenBSD have stupid stdout handling")
+#if defined(__GLIBC__) || defined(__UCLIBC__)
+# include <glob.h>
+#endif
+
+#if defined(__GLIBC__) || defined(__UCLIBC__) || defined(__NetBSD__)
+# define __PAX_UTILS_DEFAULT_LD_CACHE_CONFIG "/etc/ld.so.conf"
+#elif defined(__FreeBSD__) || defined(__DragonFly__)
+# include <elf-hints.h>
+# define __PAX_UTILS_DEFAULT_LD_CACHE_CONFIG _PATH_ELF_HINTS
 #else
-# define SET_STDOUT(fp) stdout = fp
+# undef __PAX_UTILS_DEFAULT_LD_CACHE_CONFIG
+#endif
+
+/* bounds checking code will fart on free(NULL) even though that
+ * is valid usage.  So let's wrap it if need be. */
+#ifdef __BOUNDS_CHECKING_ON
+# define free(ptr) do { if (ptr) free(ptr); } while (0)
+# define __PAX_UTILS_CLEANUP
 #endif
 
 #if !defined(bswap_16)
