@@ -16,12 +16,15 @@
 #include "paxinc.h"
 
 #ifdef WANT_SYSCAP
-#undef _POSIX_SOURCE
-#include <sys/capability.h>
+# undef _POSIX_SOURCE
+# include <sys/capability.h>
+# define WRAP_SYSCAP(x) x
+#else
+# define WRAP_SYSCAP(x)
 #endif
 
 #define PROC_DIR "/proc"
-static const char *rcsid = "$Id: pspax.c,v 1.28 2006/02/27 18:19:10 solar Exp $";
+static const char *rcsid = "$Id: pspax.c,v 1.31 2006/08/12 16:47:52 solar Exp $";
 #define argv0 "pspax"
 
 
@@ -31,6 +34,12 @@ static char show_all = 0;
 static char verbose = 0;
 static char show_banner = 1;
 static char show_phdr = 0;
+static char noexec = 1;
+static char writeexec = 1;
+
+static pid_t show_pid = 0;
+static uid_t show_uid = -1;
+static gid_t show_gid = -1;
 
 static char *get_proc_name(pid_t pid)
 {
@@ -125,7 +134,16 @@ static int print_executable_mappings(pid_t pid)
 	return 0;
 }
 
-static struct passwd *get_proc_uid(pid_t pid)
+#ifdef __BOUNDS_CHECKING_ON
+#define NOTE_TO_SELF warn( \
+	"This is bullshit but getpwuid() is leaking memory and I wasted a few hrs 1 day tracking it down in pspax\n" \
+	"Later on I forgot I tracked it down before and saw pspax leaking memory so I tracked it down all over again (silly me)\n" \
+	"Hopefully the getpwuid()/nis/nss/pam or whatever wont suck later on in the future.")
+#else
+ #define NOTE_TO_SELF
+#endif
+
+static struct passwd *get_proc_passwd(pid_t pid)
 {
 	struct stat st;
 	struct passwd *pwd;
@@ -133,14 +151,6 @@ static struct passwd *get_proc_uid(pid_t pid)
 
 	snprintf(str, sizeof(str), PROC_DIR "/%u/stat", pid);
 
-	/* this is bullshit but getpwuid() is leaking memory
-	 * and I've wasted a few hrs 1 day tracking it down.
-	 * I forgot I tracked it down before and saw pspax leaking
-	 * memory so I tracked it down again (silly me)
-	 * anyway.. please leave this comment here so I don't waste my
-	 * time again the next time I forget.
-	 * and till such time as getpwuid()/nis/nss/pam or whatever does not suck.
-	 */
 	if ((stat(str, &st)) != (-1))
 		if ((pwd = getpwuid(st.st_uid)) != NULL)
 			return pwd;
@@ -159,12 +169,12 @@ static char *get_proc_status(pid_t pid, const char *name)
 
 	len = strlen(name);
 	while (fgets(str, sizeof(str), fp)) {
-		if (strncasecmp(str, name, len) == 0) {
-			if (str[len] == ':') {
-				fclose(fp);
-				str[strlen(str) - 1] = 0;
-				return (str + len + 2);
-			}
+		if (strncasecmp(str, name, len) != 0)
+			continue;
+		if (str[len] == ':') {
+			fclose(fp);
+			str[strlen(str) - 1] = 0;
+			return (str + len + 2);
 		}
 	}
 	fclose(fp);
@@ -256,21 +266,19 @@ static const char *get_proc_phdr(pid_t pid)
 }
 
 
-static void pspax(pid_t ppid, const char *find_name)
+static void pspax(const char *find_name)
 {
 	register DIR *dir;
 	register struct dirent *de;
 	pid_t pid;
+	pid_t ppid = show_pid;
 	int have_attr, wx;
-	struct passwd *uid;
+	struct passwd *pwd;
 	struct stat st;
 	const char *pax, *type, *name, *caps, *attr;
-#ifdef WANT_SYSCAP
-	ssize_t length;
-	cap_t cap_d;
+	WRAP_SYSCAP(ssize_t length; cap_t cap_d;);
 
-	cap_d = cap_init();
-#endif
+	WRAP_SYSCAP(cap_d = cap_init());
 
 	caps = NULL;
 
@@ -303,22 +311,37 @@ static void pspax(pid_t ppid, const char *find_name)
 			if (((ppid > 0) && (pid != ppid)) || (!pid))
 				continue;
 
-#ifdef WANT_SYSCAP
-			/* this is a non-POSIX function */
-			capgetp(pid, cap_d);
-			caps = cap_to_text(cap_d, &length);
-#endif
+			wx = get_proc_maps(pid);
 
-			uid  = get_proc_uid(pid);
+			if (noexec != writeexec) {
+				if ((wx == 1) && (writeexec != wx))
+					goto next_pid;
+
+				if ((wx == 0) && (writeexec))
+					goto next_pid;
+			}
+
+			pwd  = get_proc_passwd(pid);
 			pax  = get_proc_status(pid, "PAX");
-			wx   = get_proc_maps(pid);
 			type = get_proc_type(pid);
 			name = get_proc_name(pid);
 			attr = (have_attr ? get_pid_attr(pid) : NULL);
 
+			if (show_uid != (-1) && pwd)
+				if (pwd->pw_uid != show_uid)
+					continue;
+
+			if (show_gid != (-1) && pwd)
+				if (pwd->pw_gid != show_gid)
+					continue;
+
+			/* this is a non-POSIX function */
+			WRAP_SYSCAP(capgetp(pid, cap_d));
+			WRAP_SYSCAP(caps = cap_to_text(cap_d, &length));
+
 			if (show_all || type) {
 				printf("%-8s %-6d %-6s %-4s %-10s %-16s %-4s %s %s\n",
-				       uid  ? uid->pw_name : "--------",
+				       pwd  ? pwd->pw_name : "--------",
 				       pid,
 				       pax  ? pax  : "---",
 				       (wx == 1) ? "w|x" : (wx == -1) ? "---" : "w^x",
@@ -329,10 +352,11 @@ static void pspax(pid_t ppid, const char *find_name)
 				if (verbose && wx)
 					print_executable_mappings(pid);
 			}
-#ifdef WANT_SYSCAP
-			if (caps)
-				cap_free((void *)caps);
-#endif
+
+			WRAP_SYSCAP(if (caps) cap_free((void *)caps));
+
+		next_pid:
+			continue;
 		}
 	}
 	closedir(dir);
@@ -341,12 +365,16 @@ static void pspax(pid_t ppid, const char *find_name)
 
 
 /* usage / invocation handling functions */
-#define PARSE_FLAGS "aep:vBhV"
+#define PARSE_FLAGS "aep:u:g:nwvBhV"
 #define a_argument required_argument
 static struct option const long_opts[] = {
 	{"all",       no_argument, NULL, 'a'},
 	{"header",    no_argument, NULL, 'e'},
 	{"pid",        a_argument, NULL, 'p'},
+	{"user",       a_argument, NULL, 'u'},
+	{"group",      a_argument, NULL, 'g'},
+	{"nx",        no_argument, NULL, 'n'},
+	{"wx",        no_argument, NULL, 'w'},
 	{"verbose",   no_argument, NULL, 'v'},
 	{"nobanner",  no_argument, NULL, 'B'},
 	{"help",      no_argument, NULL, 'h'},
@@ -357,6 +385,10 @@ static const char *opts_help[] = {
 	"Show all processes",
 	"Print GNU_STACK/PT_LOAD markings",
 	"Process ID/pid #",
+	"Process UID #",
+	"Process GID #",
+	"Only display w^x processes",
+	"Only display w|x processes",
 	"Be verbose about executable mappings",
 	"Don't display the header",
 	"Print this help and exit",
@@ -383,10 +415,9 @@ static void usage(int status)
 }
 
 /* parse command line arguments and preform needed actions */
-static pid_t parseargs(int argc, char *argv[])
+static void parseargs(int argc, char *argv[])
 {
 	int flag;
-	pid_t pid = 0;
 	opterr = 0;
 	while ((flag=getopt_long(argc, argv, PARSE_FLAGS, long_opts, NULL)) != -1) {
 		switch (flag) {
@@ -402,15 +433,16 @@ static pid_t parseargs(int argc, char *argv[])
 		case 'B': show_banner = 0; break;
 		case 'a': show_all = 1; break;
 		case 'e': show_phdr = 1; break;
-		case 'p': pid = atoi(optarg); break;
+		case 'p': show_pid = atoi(optarg); break;
+		case 'u': show_uid = atoi(optarg); break;
+		case 'g': show_gid = atoi(optarg); break;
+		case 'n': noexec = 1; writeexec = 0; break;
+		case 'w': noexec = 0; writeexec = 1; break;
 		case 'v': verbose++; break;
 
 		case ':':
-			warn("Option missing parameter");
-			usage(EXIT_FAILURE);
-			break;
 		case '?':
-			warn("Unknown option");
+			warn("Unknown option or missing parameter");
 			usage(EXIT_FAILURE);
 			break;
 		default:
@@ -418,19 +450,21 @@ static pid_t parseargs(int argc, char *argv[])
 			break;
 		}
 	}
-	return pid;
 }
 
 
 
 int main(int argc, char *argv[])
 {
-	pid_t pid = parseargs(argc, argv);
 	char *name = NULL;
 
-	if ((optind < argc) && (pid == 0))
+	parseargs(argc, argv);
+
+	if ((optind < argc) && (show_pid == 0))
 		name = argv[optind];
 
-	pspax(pid, name);
+	pspax(name);
+
+	NOTE_TO_SELF;
 	return EXIT_SUCCESS;
 }
