@@ -1,7 +1,7 @@
 /*
  * Copyright 2003-2006 Gentoo Foundation
  * Distributed under the terms of the GNU General Public License v2
- * $Header: /var/cvsroot/gentoo-projects/pax-utils/scanelf.c,v 1.153 2006/05/17 21:45:20 solar Exp $
+ * $Header: /var/cvsroot/gentoo-projects/pax-utils/scanelf.c,v 1.158 2006/08/25 06:32:08 vapier Exp $
  *
  * Copyright 2003-2006 Ned Ludd        - <solar@gentoo.org>
  * Copyright 2004-2006 Mike Frysinger  - <vapier@gentoo.org>
@@ -9,7 +9,7 @@
 
 #include "paxinc.h"
 
-static const char *rcsid = "$Id: scanelf.c,v 1.153 2006/05/17 21:45:20 solar Exp $";
+static const char *rcsid = "$Id: scanelf.c,v 1.158 2006/08/25 06:32:08 vapier Exp $";
 #define argv0 "scanelf"
 
 #define IS_MODIFIER(c) (c == '%' || c == '#' || c == '+')
@@ -85,74 +85,6 @@ caddr_t ldcache = 0;
 size_t ldcache_size = 0;
 unsigned long setpax = 0UL;
 
-/* utility funcs */
-static char *xstrdup(const char *s)
-{
-	char *ret = strdup(s);
-	if (!ret) err("Could not strdup(): %s", strerror(errno));
-	return ret;
-}
-static void *xmalloc(size_t size)
-{
-	void *ret = malloc(size);
-	if (!ret) err("Could not malloc() %li bytes", (unsigned long)size);
-	return ret;
-}
-static void *xrealloc(void *ptr, size_t size)
-{
-	void *ret = realloc(ptr, size);
-	if (!ret) err("Could not realloc() %li bytes", (unsigned long)size);
-	return ret;
-}
-static void xstrncat(char **dst, const char *src, size_t *curr_len, size_t n)
-{
-	size_t new_len;
-
-	new_len = strlen(*dst) + strlen(src);
-	if (*curr_len <= new_len) {
-		*curr_len = new_len + (*curr_len / 2);
-		*dst = realloc(*dst, *curr_len);
-		if (!*dst)
-			err("could not realloc() %li bytes", (unsigned long)*curr_len);
-	}
-
-	if (n)
-		strncat(*dst, src, n);
-	else
-		strcat(*dst, src);
-}
-static inline void xchrcat(char **dst, const char append, size_t *curr_len)
-{
-	static char my_app[2];
-	my_app[0] = append;
-	my_app[1] = '\0';
-	xstrcat(dst, my_app, curr_len);
-}
-
-/* Match filename against entries in matchlist, return TRUE
- * if the file is listed */
-static int file_matches_list(const char *filename, char **matchlist)
-{
-	char **file;
-	char *match;
-	char buf[__PAX_UTILS_PATH_MAX];
-
-	if (matchlist == NULL)
-		return 0;
-
-	for (file = matchlist; *file != NULL; file++) {
-		if (search_path) {
-			snprintf(buf, sizeof(buf), "%s%s", search_path, *file);
-			match = buf;
-		} else {
-			match = *file;
-		}
-		if (fnmatch(match, filename, 0) == 0)
-			return 1;
-	}
-	return 0;
-}
-
 
 
 /* sub-funcs for scanelf_file() */
@@ -336,6 +268,8 @@ static char *scanelf_file_phdr(elfobj *elf, char *found_phdr, char *found_relro,
 		} \
 		skip_this_shdr##B: \
 		if (!multi_stack) { \
+			if (file_matches_list(elf->filename, qa_execstack)) \
+				return NULL; \
 			*found_phdr = 1; \
 			shown = 1; \
 			memcpy(ret, "!WX", 3); \
@@ -512,7 +446,7 @@ static void rpath_security_checks(elfobj *elf, char *item, const char *dt_type)
 		case '$':
 			if (fstat(elf->fd, &st) != -1)
 				if ((st.st_mode & S_ISUID) || (st.st_mode & S_ISGID))
-					warnf("Security problem with %s='%s' in %s with mode set of %o", 
+					warnf("Security problem with %s='%s' in %s with mode set of %o",
 					      dt_type, item, elf->filename, st.st_mode & 07777);
 			break;
 		default:
@@ -750,7 +684,31 @@ static char *lookup_cache_lib(elfobj *elf, char *fname)
 	}
 	return buf;
 }
+#elif defined(__NetBSD__)
+static char *lookup_cache_lib(elfobj *elf, char *fname)
+{
+	static char buf[__PAX_UTILS_PATH_MAX] = "";
+	static struct stat st;
 
+	char **ldpath;
+	for (ldpath = ldpaths; *ldpath != NULL; ldpath++) {
+		if ((unsigned) snprintf(buf, sizeof(buf), "%s/%s", *ldpath, fname) >= sizeof(buf))
+			continue; /* if the pathname is too long, or something went wrong, ignore */
+
+		if (stat(buf, &st) != 0)
+			continue; /* if the lib doesn't exist in *ldpath, look further */
+
+		/* NetBSD doesn't actually do sanity checks, it just loads the file
+		 * and if that doesn't work, continues looking in other directories.
+		 * This cannot easily be safely emulated, unfortunately. For now,
+		 * just assume that if it exists, it's a valid library. */
+
+		return buf;
+	}
+
+	/* not found in any path */
+	return NULL;
+}
 #else
 #warning Cache support not implemented for your target
 static char *lookup_cache_lib(elfobj *elf, char *fname)
@@ -970,8 +928,9 @@ static char *scanelf_file_sym(elfobj *elf, char *found_sym)
 					*found_sym = 1; \
 				} else { \
 					/* allow the user to specify a comma delimited list of symbols to search for */ \
-					char *this_sym, *next_sym; \
+					char *this_sym, *this_sym_ver, *next_sym; \
 					this_sym = ret; \
+					this_sym_ver = versioned_symname; \
 					do { \
 						next_sym = strchr(this_sym, ','); \
 						if (next_sym == NULL) \
@@ -981,15 +940,17 @@ static char *scanelf_file_sym(elfobj *elf, char *found_sym)
 							if (sym->st_shndx == SHN_UNDEF) \
 								goto skip_this_sym##B; \
 							++this_sym; \
+							++this_sym_ver; \
 						/* do we want an undefined symbol ? */ \
 						} else if (*this_sym == '-') { \
 							if (sym->st_shndx != SHN_UNDEF) \
 								goto skip_this_sym##B; \
 							++this_sym; \
+							++this_sym_ver; \
 						} \
 						/* ok, lets compare the name now */ \
 						if ((strncmp(this_sym, symname, (next_sym-this_sym)) == 0 && symname[next_sym-this_sym] == '\0') || \
-						    (strncmp(symname, versioned_symname, strlen(versioned_symname)) == 0)) { \
+						    (strncmp(this_sym_ver, symname, strlen(this_sym_ver)) == 0)) { \
 							if (be_semi_verbose) { \
 								char buf[126]; \
 								snprintf(buf, sizeof(buf), "%lX %s %s", \
@@ -1373,7 +1334,7 @@ static int scanelf_from_file(const char *filename)
 	return 0;
 }
 
-#if defined(__GLIBC__) || defined(__UCLIBC__)
+#if defined(__GLIBC__) || defined(__UCLIBC__) || defined(__NetBSD__)
 
 static int load_ld_cache_config(int i, const char *fname)
 {
@@ -1392,6 +1353,7 @@ static int load_ld_cache_config(int i, const char *fname)
 			*p = 0;
 		if ((p = strchr(path, '\n')) != NULL)
 			*p = 0;
+#ifdef __linux__
 		// recursive includes of the same file will make this segfault.
 		if ((memcmp(path, "include", 7) == 0) && isblank(path[7])) {
 			glob64_t gl;
@@ -1421,6 +1383,7 @@ static int load_ld_cache_config(int i, const char *fname)
 			} else
 				abort();
 		}
+#endif
 		if (*path != '/')
 			continue;
 
@@ -1472,7 +1435,7 @@ static int load_ld_cache_config(int i, const char *fname)
 			break;
 	}
 	ldpaths[i] = NULL;
-	
+
 	free(b);
 	fclose(fp);
 	return i;
@@ -1616,7 +1579,7 @@ static void usage(int status)
 	printf("Options: -[%s]\n", PARSE_FLAGS);
 	for (i = 0; long_opts[i].name; ++i)
 		if (long_opts[i].has_arg == no_argument)
-			printf("  -%c, --%-14s* %s\n", long_opts[i].val, 
+			printf("  -%c, --%-14s* %s\n", long_opts[i].val,
 			       long_opts[i].name, opts_help[i]);
 		else
 			printf("  -%c, --%-7s <arg> * %s\n", long_opts[i].val,
@@ -1624,7 +1587,7 @@ static void usage(int status)
 
 	if (status != EXIT_SUCCESS)
 		exit(status);
-	
+
 	puts("\nThe format modifiers for the -F option are:");
 	puts(" F Filename \tx PaX Flags \te STACK/RELRO");
 	puts(" t TEXTREL  \tr RPATH     \tn NEEDED");
@@ -1918,3 +1881,73 @@ int main(int argc, char *argv[])
 #endif
 	return EXIT_SUCCESS;
 }
+
+
+
+/* utility funcs */
+static char *xstrdup(const char *s)
+{
+	char *ret = strdup(s);
+	if (!ret) err("Could not strdup(): %s", strerror(errno));
+	return ret;
+}
+static void *xmalloc(size_t size)
+{
+	void *ret = malloc(size);
+	if (!ret) err("Could not malloc() %li bytes", (unsigned long)size);
+	return ret;
+}
+static void *xrealloc(void *ptr, size_t size)
+{
+	void *ret = realloc(ptr, size);
+	if (!ret) err("Could not realloc() %li bytes", (unsigned long)size);
+	return ret;
+}
+static void xstrncat(char **dst, const char *src, size_t *curr_len, size_t n)
+{
+	size_t new_len;
+
+	new_len = strlen(*dst) + strlen(src);
+	if (*curr_len <= new_len) {
+		*curr_len = new_len + (*curr_len / 2);
+		*dst = realloc(*dst, *curr_len);
+		if (!*dst)
+			err("could not realloc() %li bytes", (unsigned long)*curr_len);
+	}
+
+	if (n)
+		strncat(*dst, src, n);
+	else
+		strcat(*dst, src);
+}
+static inline void xchrcat(char **dst, const char append, size_t *curr_len)
+{
+	static char my_app[2];
+	my_app[0] = append;
+	my_app[1] = '\0';
+	xstrcat(dst, my_app, curr_len);
+}
+
+/* Match filename against entries in matchlist, return TRUE
+ * if the file is listed */
+static int file_matches_list(const char *filename, char **matchlist)
+{
+	char **file;
+	char *match;
+	char buf[__PAX_UTILS_PATH_MAX];
+
+	if (matchlist == NULL)
+		return 0;
+
+	for (file = matchlist; *file != NULL; file++) {
+		if (search_path) {
+			snprintf(buf, sizeof(buf), "%s%s", search_path, *file);
+			match = buf;
+		} else {
+			match = *file;
+		}
+		if (fnmatch(match, filename, 0) == 0)
+			return 1;
+	}
+	return 0;
+}
