@@ -1,25 +1,12 @@
 /*
- * Copyright 2003-2005 Gentoo Foundation
+ * Copyright 2003-2006 Gentoo Foundation
  * Distributed under the terms of the GNU General Public License v2
  * $Header: /var/cvsroot/gentoo-projects/pax-utils/scanmach.c,v 1.88 2005/09/30 03:30:54 vapier Exp $
  *
- * Copyright 2003-2005 Ned Ludd        - <solar@gentoo.org>
- * Copyright 2004-2005 Mike Frysinger  - <vapier@gentoo.org>
+ * Copyright 2003-2006 Ned Ludd        - <solar@gentoo.org>
+ * Copyright 2004-2006 Mike Frysinger  - <vapier@gentoo.org>
  */
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <sys/types.h>
-#include <libgen.h>
-#include <limits.h>
-#define __USE_GNU
-#include <string.h>
-#include <errno.h>
-#include <unistd.h>
-#include <sys/stat.h>
-#include <dirent.h>
-#include <getopt.h>
-#include <assert.h>
 #include "paxinc.h"
 
 static const char *rcsid = "$Id: scanmach.c,v 1.88 2005/09/30 03:30:54 vapier Exp $";
@@ -239,7 +226,7 @@ static void scanmacho_dir(const char *path)
 	register DIR *dir;
 	register struct dirent *dentry;
 	struct stat st_top, st;
-	char buf[_POSIX_PATH_MAX];
+	char buf[__PAX_UTILS_PATH_MAX];
 	size_t pathlen = 0, len = 0;
 
 	/* make sure path exists */
@@ -288,14 +275,14 @@ static int scanmacho_from_file(char *filename)
 {
 	FILE *fp = NULL;
 	char *p;
-	char path[_POSIX_PATH_MAX];
+	char path[__PAX_UTILS_PATH_MAX];
 
 	if (((strcmp(filename, "-")) == 0) && (ttyname(0) == NULL))
 		fp = stdin;
 	else if ((fp = fopen(filename, "r")) == NULL)
 		return 1;
 
-	while ((fgets(path, _POSIX_PATH_MAX, fp)) != NULL) {
+	while ((fgets(path, __PAX_UTILS_PATH_MAX, fp)) != NULL) {
 		if ((p = strchr(path, '\n')) != NULL)
 			*p = 0;
 		search_path = path;
@@ -310,13 +297,13 @@ static void load_ld_so_conf()
 {
 	FILE *fp = NULL;
 	char *p;
-	char path[_POSIX_PATH_MAX];
+	char path[__PAX_UTILS_PATH_MAX];
 	int i = 0;
 
 	if ((fp = fopen("/etc/ld.so.conf", "r")) == NULL)
 		return;
 
-	while ((fgets(path, _POSIX_PATH_MAX, fp)) != NULL) {
+	while ((fgets(path, __PAX_UTILS_PATH_MAX, fp)) != NULL) {
 		if (*path != '/')
 			continue;
 
@@ -496,10 +483,8 @@ static void parseargs(int argc, char *argv[])
 			from_file = xstrdup(optarg);
 			break;
 		case 'o': {
-			FILE *fp = NULL;
-			if ((fp = freopen(optarg, "w", stdout)) == NULL)
+			if (freopen(optarg, "w", stdout) == NULL)
 				err("Could not open output stream '%s': %s", optarg, strerror(errno));
-			SET_STDOUT(fp);
 			break;
 		}
 
